(function (global) {
    "use strict";

    global.FRCC = global.FRCC || {};
    global.FRCC.Portal = global.FRCC.Portal || {};
    global.FRCC.Portal.Conditions = (function () {

        var setupConditions = function (conditionConfiguration) {
            if (conditionConfiguration.HiddenFields?.length) {
                showHideFields(false, ...conditionConfiguration.HiddenFields);
            }

            var showHideConditionsByTarget = {};

            conditionConfiguration.Conditions.forEach((condition, index) => {
                switch (condition.ConditionType) {
                    case 643260000: /* Show Field Based on Condition */
                        condition.TargetFields.forEach(targetField => {
                            if (!showHideConditionsByTarget[targetField.FieldName]) {
                                showHideConditionsByTarget[targetField.FieldName] = {
                                    TargetField: targetField,
                                    Conditions: []
                                }
                            }

                            showHideConditionsByTarget[targetField.FieldName].Conditions.push(condition);
                        });
                        
                        break;
                    case 643260001: /* Enable / Disable Field Based on Condition */
                        enableDisableFieldBasedOnCondition(condition);
                        addOnChange(condition.ConditionExpressions,
                            () => enableDisableFieldBasedOnCondition(condition));
                        break;

                    case 643260002: /* Set Value Based on Condition */
                        setValueBasedOnCondition(condition);
                        addOnChange(condition.ConditionExpressions,
                            () => setValueBasedOnCondition(condition),
                            ...condition.TargetFields.map(f => f.FieldName));
                }
            });

            Object.values(showHideConditionsByTarget).forEach((showHideConditionByTarget) => {
                showHideFieldBasedOnConditions(showHideConditionByTarget.TargetField, showHideConditionByTarget.Conditions);

                var allExpressions = showHideConditionByTarget.Conditions
                    .reduce((result, condition) => result.concat(condition.ConditionExpressions), []);
                addOnChange(allExpressions,
                    () => showHideFieldBasedOnConditions(showHideConditionByTarget.TargetField, showHideConditionByTarget.Conditions));
            })
        };

        var showHideFieldBasedOnConditions = function (targetField, conditions) {
            if (conditions.some(condition => evaluateCondition(condition))) {
                $(`#${targetField.FieldName}`).closest('.cell').show();
            }
            else {
                $(`#${targetField.FieldName}`).closest('.cell').hide();
                clearFieldBasedOnType(targetField.FieldName, targetField.FieldType);
            }
        }

        var showHideFields = function (shouldShow, ...fieldNames) {
            if (shouldShow) {
                $("#" + fieldNames.join(', #')).closest('.cell').show();
            } else {
                $("#" + fieldNames.join(', #')).closest('.cell').hide();
            }
        }

        var setValueBasedOnCondition = function (condition) {
            if (evaluateCondition(condition)) {
                condition.TargetFields.forEach(f =>
                    $(`#${f.FieldName}`).val(condition.TargetValue || ""));
            }
        }

        var enableDisableFieldBasedOnCondition = function (condition) {
            var isEnabled = evaluateCondition(condition);

            condition.TargetFields.forEach(targetField => {
                $(`#${targetField.FieldName}`).attr('readonly', !isEnabled);
				$(`#${targetField.FieldName} input`).attr('readonly', !isEnabled);
                $(`select#${targetField.FieldName}`).attr('disabled', !isEnabled);

                if (isEnabled) {
                    $(`#${targetField.FieldName}`).removeClass('readonly');
                    $(`#${targetField.FieldName} + .text-muted`).hide();
					
					$(`#${targetField.FieldName} input`).removeClass('readonly');
                    $(`#${targetField.FieldName} + .text-muted`).hide();
                } else {
                    $(`#${targetField.FieldName}`).addClass('readonly');
					$(`#${targetField.FieldName} input`).addClass('readonly');
                    $(`#${targetField.FieldName} + .text-muted`).show();
                }

                switch (targetField.FieldType) {
                    // Choice
                    case 643_260_006:
                        // Multi choice
                        const multiChoiceContainer = $(`#${targetField.FieldName}`).parent().find('.msos-container');
                        if (multiChoiceContainer.length) {
                            if (isEnabled) { multiChoiceContainer.removeClass('msos-disabled'); }
                            else { multiChoiceContainer.addClass('msos-disabled'); }
                        }

                        break;
                }
            });
        }

        var clearFieldBasedOnType = function (fieldName, fieldType) {
            if (isFieldRequired(fieldName)) { return; }

            switch (fieldType) {
                // Single-line
                case 643260000:
                    $(`#${fieldName}`).val('');
                    break;
                default:
                    break;
            }
        }

        var isFieldRequired = function (fieldName) {
            return Page_Validators.some(e => e.id == `RequiredFieldValidator${fieldName}`);
        }

        var evaluateCondition = function (condition) {
            if (condition.IsOr) {
                return condition.ConditionExpressions.some(expression => evaluateConditionExpression(expression));
            }
            else {
                return condition.ConditionExpressions.every(expression => evaluateConditionExpression(expression));
            }
        }

        var evaluateConditionExpression = function (conditionExpression) {
            var bReturn = false;
            var rightValue = conditionExpression.ConditionValue || "";
            var leftValue = "";
            switch (conditionExpression.ConditionField.FieldType) {
                case 643_260_007: // Yes or No
                    leftValue = $(`#${conditionExpression.ConditionField.FieldName} :checked`)[0]?.value || "";
                    break;
                case 643_260_006: // Choice
                    var inputControl = $(`#${conditionExpression.ConditionField.FieldName}`)[0] ;
                    leftValue = inputControl.value || "";
                    break;
                default:
                    var inputControl = $(`#${conditionExpression.ConditionField.FieldName}`)[0];
                    leftValue = inputControl.value || "";
            }
        
            //choice with multiple values
            if(conditionExpression.ConditionField.FieldType == 643_260_006 && rightValue.indexOf(',') > -1 )
            {
                try {
                    var sourceValues = leftValue
                        ? JSON.parse(leftValue).map(v => v.Label.LocalizedLabels[0].Label.toLowerCase())
                        : [];
                    var conditionValues = rightValue.toLowerCase().split(",");

                    return sourceValues.length == conditionValues.length
                        && (sourceValues.length > 0 && sourceValues.every(v => conditionValues.includes(v)));
                }
                catch {
                    return false;
                }
            }
            else
            {
                switch (conditionExpression.ConditionOperator) {
                    case 643260002://greater
                        bReturn = eval(Number(leftValue) > Number(rightValue));
                        break; 
                    case 643260003: //less
                        bReturn = eval(Number(leftValue) < Number(rightValue));
                        break;
                    case 643260004: //greated than equal to
                        bReturn = eval(Number(leftValue) >= Number(rightValue));
                        break;
                    case 643260005: //lessthan equal to
                        bReturn = eval(Number(leftValue) <= Number(rightValue));
                        break;
                    case 643260006: //Contains
                        leftValue = leftValue.toString().toLowerCase();
                        rightValue = rightValue.toString().toLowerCase();
                        bReturn = eval(leftValue.indexOf(rightValue) > -1);
                        break;
                    case 643260007: //Not Contains
                        leftValue = leftValue.toString().toLowerCase();
                        rightValue = rightValue.toString().toLowerCase();
                        bReturn = eval(leftValue.indexOf(rightValue) == -1);
                        break;
                    case 643260001: //not equal to
                        leftValue = leftValue.toString().toLowerCase();
                        rightValue = rightValue.toString().toLowerCase();
                        bReturn = eval(String(leftValue) != String(rightValue));
                        break;
                    case 643_260_000: // Equal
                    default:
                        leftValue = leftValue.toString().toLowerCase();
                        rightValue = rightValue.toString().toLowerCase();
                        bReturn = eval(String(leftValue) == String(rightValue));
                        break;
                }
                return bReturn;
            }
        }

        var addOnChange = function (conditionExpressions, handler, ...excludeTriggerFields) {
            if (!conditionExpressions || !conditionExpressions.length) { return; }

            conditionExpressions.forEach(conditionExpression => {
                if (excludeTriggerFields.includes(conditionExpression.ConditionField.FieldName)) {
                    return;
                }

                var inputControl = $(`#${conditionExpression.ConditionField.FieldName}`)[0];
                if(inputControl)
                {
                    if (inputControl.type === 'hidden') {
                        /* Hidden inputs, like those for multiselect controls, don't trigger on change, so
                        * we'll need to use a MutationObserver instead */
                        new MutationObserver(handler)
                            .observe(inputControl, { attributes: true, attributeFilter: ['value'] });
                    } else {
                        inputControl.addEventListener('change', handler, false);
                        
                    }
                }
            });
        }

        return {
            setupConditions: setupConditions,
            showHideFields: showHideFields
        }
    })();

})(this);